# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tools for making the most of S3Api metadata."""


import base64
import binascii
import copy
import re

from googlecloudsdk.api_lib.storage import metadata_util
from googlecloudsdk.api_lib.storage import xml_metadata_field_converters
from googlecloudsdk.command_lib.storage import errors
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage import user_request_args_factory
from googlecloudsdk.command_lib.storage.resources import gcs_resource_reference
from googlecloudsdk.command_lib.storage.resources import resource_reference
from googlecloudsdk.command_lib.storage.resources import s3_resource_reference
from googlecloudsdk.core import log

_USER_METADATA_FIELDS = frozenset([
    'CacheControl',
    'ContentDisposition',
    'ContentEncoding',
    'ContentLanguage',
    'ContentType',
    'Metadata',
])

_NON_USER_METADATA_FIELDS = frozenset([
    'ACL',
    'AccessControlPolicy',
    'StorageClass',
])

_COMMON_S3_METADATA_FIELDS = frozenset().union(
    *[_USER_METADATA_FIELDS, _NON_USER_METADATA_FIELDS]
)

_S3_TO_GENERIC_FIELD_NAMES_ = {
    'CacheControl': 'cache_control',
    'ContentDisposition': 'content_disposition',
    'ContentEncoding': 'content_encoding',
    'ContentLanguage': 'content_language',
    'ContentType': 'content_type',
    'ContentMD5': 'md5_hash',
    'StorageClass': 'storage_class',
}

_GCS_TO_S3_PREDEFINED_ACL_TRANSLATION_DICT = {
    'authenticatedRead': 'authenticated-read',
    'bucketOwnerFullControl': 'bucket-owner-full-control',
    'bucketOwnerRead': 'bucket-owner-read',
    'private': 'private',
    'publicRead': 'public-read',
    'publicReadWrite': 'public-read-write',
}
_SCHEME_TO_BUCKET_RESOURCE_DICT = {
    storage_url.ProviderPrefix.GCS: gcs_resource_reference.GcsBucketResource,
    storage_url.ProviderPrefix.S3: s3_resource_reference.S3BucketResource,
}
_SCHEME_TO_OBJECT_RESOURCE_DICT = {
    storage_url.ProviderPrefix.GCS: gcs_resource_reference.GcsObjectResource,
    storage_url.ProviderPrefix.S3: s3_resource_reference.S3ObjectResource,
}
# Determines whether an etag is a valid MD5.
MD5_REGEX = re.compile(r'^[a-fA-F0-9]{32}$')


def get_acl_policy_with_added_and_removed_grants(acl_dict, request_config):
  """Returns full ACL policy object with requested changes.

  Args:
    acl_dict (dict): Contains S3-format ACL policy dict for bucket or object.
      Usually of the form: "{"Grants": [...], "Owner": {...}}". See:
      https://boto3.amazonaws.com/v1/documentation/api/latest/reference
      /services/s3.html#S3.Client.get_bucket_acl
    request_config (request_config_factory._RequestConfig): Contains desired
      changes for the ACL policy.

  Returns:
    dict: Deep copy of acl_dict with added and removed grants and
      removed "ResponseMetadata" field to allow for reuse in PUT API calls.
  """
  # Avoid unexpected destructive side effect of popping to original acl_dict.
  acl_dict_copy = copy.deepcopy(acl_dict)
  # Unnecessary extra metadata that we need to remove for PUT calls.
  acl_dict_copy.pop('ResponseMetadata', None)

  if not request_config.resource_args:
    # No ACL grants to add or remove.
    return acl_dict_copy

  acl_grants_to_add = request_config.resource_args.acl_grants_to_add
  if acl_grants_to_add:
    for new_grant in acl_grants_to_add:
      # Avoid unexpected destructive side effect of pop to request_config.
      new_grant_copy = copy.deepcopy(new_grant)
      # Input format flattens Grantee fields for UX.
      permission = new_grant_copy.pop('Permission')
      acl_dict_copy['Grants'].append({
          'Permission': permission,
          'Grantee': new_grant_copy,
      })

  acl_grants_to_remove = request_config.resource_args.acl_grants_to_remove
  if acl_grants_to_remove:
    entity_identifiers_to_remove = set(acl_grants_to_remove)
    filtered_grants = []
    for existing_grant in acl_dict_copy.get('Grants', []):
      existing_grantee = existing_grant.get('Grantee', {})
      existing_grantee_identifiers = set(
          (existing_grantee.get('EmailAddress'), existing_grantee.get('ID'),
           existing_grantee.get('URI')))
      if not entity_identifiers_to_remove.intersection(
          existing_grantee_identifiers):
        filtered_grants.append(existing_grant)
    acl_dict_copy['Grants'] = filtered_grants

  return acl_dict_copy


def _copy_metadata(source_metadata_dict, destination_metadata_dict, fields):
  """Copy fields(provided in arguments) from one metadata dict to another."""
  if not destination_metadata_dict:
    destination_metadata_dict = {}
  if not source_metadata_dict:
    return destination_metadata_dict
  for field in fields:
    if field in source_metadata_dict:
      destination_metadata_dict[field] = source_metadata_dict[field]
  return destination_metadata_dict


def copy_object_metadata(source_metadata_dict, destination_metadata_dict):
  """Copies common S3 fields from one metadata dict to another."""
  return _copy_metadata(
      source_metadata_dict,
      destination_metadata_dict,
      _COMMON_S3_METADATA_FIELDS,
  )


def copy_user_metadata_fields(source_metadata_dict, destination_metadata_dict):
  """Copies user_metadata fields from one metadata dict to another."""
  return _copy_metadata(
      source_metadata_dict,
      destination_metadata_dict,
      _USER_METADATA_FIELDS,
  )


def translate_predefined_acl_string_to_s3(predefined_acl_string):
  """Translates Apitools predefined ACL enum key (as string) to S3 equivalent.

  Args:
    predefined_acl_string (str): Value representing user permissions.

  Returns:
    Translated ACL string.

  Raises:
    Error: Predefined ACL translation could not be found.
  """
  if predefined_acl_string not in _GCS_TO_S3_PREDEFINED_ACL_TRANSLATION_DICT:
    raise errors.Error(
        'Could not translate predefined_acl_string {} to'
        ' AWS-accepted ACL.'.format(predefined_acl_string)
    )
  return _GCS_TO_S3_PREDEFINED_ACL_TRANSLATION_DICT[predefined_acl_string]


def _get_object_url_from_xml_response(scheme,
                                      object_dict,
                                      bucket_name,
                                      object_name=None):
  """Creates storage_url.CloudUrl from S3 API response.

  Args:
    scheme (storage_url.ProviderPrefix): Prefix used for provider URLs.
    object_dict (dict): Dictionary representing S3 API response.
    bucket_name (str): Bucket to include in URL.
    object_name (str | None): Object to include in URL.

  Returns:
    storage_url.CloudUrl populated with data.
  """
  return storage_url.CloudUrl(
      scheme=scheme,
      bucket_name=bucket_name,
      resource_name=object_name,
      generation=object_dict.get('VersionId'))


def _get_etag(object_dict):
  """Returns the cleaned-up etag value, if present."""
  if 'ETag' in object_dict:
    etag = object_dict.get('ETag')
  elif 'CopyObjectResult' in object_dict:
    etag = object_dict['CopyObjectResult'].get('ETag')
  else:
    etag = None

  # The S3 API returns etag wrapped in quotes in some cases.
  if etag and etag.startswith('"') and etag.endswith('"'):
    return etag.strip('"')

  return etag


def _get_md5_hash_from_etag(etag, object_url):
  """Returns base64 encoded MD5 hash, if etag is valid MD5."""
  if etag and MD5_REGEX.match(etag):
    encoded_bytes = base64.b64encode(binascii.unhexlify(etag))
    return encoded_bytes.decode(encoding='utf-8')
  else:
    log.debug(
        'Non-MD5 etag ("%s") present for object: %s.'
        ' Data integrity checks are not possible.', etag, object_url)
  return None


def _get_crc32c_hash_from_object_dict(object_dict):
  """Returns base64 encoded CRC32C hash from object response headers."""
  response_metadata = object_dict.get('ResponseMetadata', {})
  headers = response_metadata.get('HTTPHeaders', {})
  hash_header = headers.get('x-goog-hash', '').strip()
  result = re.search(r'crc32c\=([^,]+)', hash_header)
  if result:
    return result.group(1)


def _get_error_or_value(value):
  """Returns the error string if value is error or the value itself."""
  if isinstance(value, dict) and 'ResponseMetadata' in value:
    value_copy = value.copy()
    value_copy.pop('ResponseMetadata')
    return value_copy
  return value


def get_bucket_resource_from_xml_response(scheme, bucket_dict, bucket_name):
  """Creates resource_reference.S3BucketResource from S3 API response.

  Args:
    scheme (storage_url.ProviderPrefix): Prefix used for provider URLs.
    bucket_dict (dict): Dictionary representing S3 API response.
    bucket_name (str): Bucket response is relevant to.

  Returns:
    resource_reference.S3BucketResource populated with data.
  """
  requester_pays = _get_error_or_value(bucket_dict.get('Payer'))
  if requester_pays == 'Requester':
    requester_pays = True
  elif requester_pays == 'BucketOwner':
    requester_pays = False

  versioning_enabled = _get_error_or_value(bucket_dict.get('Versioning'))
  if isinstance(versioning_enabled, dict):
    if versioning_enabled.get('Status') == 'Enabled':
      versioning_enabled = True
    else:
      versioning_enabled = None

  return _SCHEME_TO_BUCKET_RESOURCE_DICT[scheme](
      storage_url.CloudUrl(scheme, bucket_name),
      acl=_get_error_or_value(bucket_dict.get('ACL')),
      cors_config=_get_error_or_value(bucket_dict.get('CORSRules')),
      lifecycle_config=_get_error_or_value(
          bucket_dict.get('LifecycleConfiguration')),
      logging_config=_get_error_or_value(bucket_dict.get('LoggingEnabled')),
      requester_pays=requester_pays,
      location=_get_error_or_value(bucket_dict.get('LocationConstraint')),
      metadata=bucket_dict,
      versioning_enabled=versioning_enabled,
      website_config=_get_error_or_value(bucket_dict.get('Website')))


def _parse_object_tags_if_any(
    object_dict
):
  """Parse object tags from object response in format of GCS object contexts."""
  obj_tags = object_dict.get('TagSet', None)
  if not obj_tags:
    return None
  # It is guaranteed that 'Key' and 'Value' are in tag dict. See format here:
  # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3/client/get_object_tagging.html
  return {
      tag['Key']: tag['Value'] for tag in obj_tags
  }


def get_object_resource_from_xml_response(scheme,
                                          object_dict,
                                          bucket_name,
                                          object_name=None,
                                          acl_dict=None):
  """Creates resource_reference.S3ObjectResource from S3 API response.

  Args:
    scheme (storage_url.ProviderPrefix): Prefix used for provider URLs.
    object_dict (dict): Dictionary representing S3 API response.
    bucket_name (str): Bucket response is relevant to.
    object_name (str|None): Object if relevant to query.
    acl_dict (dict|None): Response from S3 get_object_acl API call.

  Returns:
    resource_reference.S3ObjectResource populated with data.
  """
  object_url = _get_object_url_from_xml_response(
      scheme, object_dict, bucket_name, object_name or object_dict['Key'])

  if 'Size' in object_dict:
    size = object_dict.get('Size')
  else:
    size = object_dict.get('ContentLength')

  encryption_algorithm = object_dict.get(
      'ServerSideEncryption', object_dict.get('SSECustomerAlgorithm'))
  etag = _get_etag(object_dict)

  if acl_dict:
    # Full ACL policy more detailed than predefined ACL string.
    raw_acl_data = acl_dict
  else:
    # Predefined ACL string or None.
    raw_acl_data = object_dict.get('ACL')
  if raw_acl_data:
    object_dict['ACL'] = raw_acl_data
  acl = _get_error_or_value(raw_acl_data)

  object_resource = _SCHEME_TO_OBJECT_RESOURCE_DICT[scheme](
      object_url,
      acl=acl,
      cache_control=object_dict.get('CacheControl'),
      component_count=object_dict.get('PartsCount'),
      content_disposition=object_dict.get('ContentDisposition'),
      content_encoding=object_dict.get('ContentEncoding'),
      content_language=object_dict.get('ContentLanguage'),
      content_type=object_dict.get('ContentType'),
      creation_time=object_dict.get('LastModified'),
      custom_fields=object_dict.get('Metadata'),
      encryption_algorithm=encryption_algorithm,
      etag=etag,
      kms_key=object_dict.get('SSEKMSKeyId'),
      md5_hash=_get_md5_hash_from_etag(etag, object_url),
      metadata=object_dict,
      size=size,
      storage_class=object_dict.get('StorageClass'),
      update_time=object_dict.get('LastModified'),
  )
  # The CRC32C is added if available only for GCS.
  if scheme == storage_url.ProviderPrefix.GCS:
    object_resource.crc32c_hash = _get_crc32c_hash_from_object_dict(object_dict)
  # The tags are added if available only for S3.
  if scheme == storage_url.ProviderPrefix.S3:
    object_resource.tags = _parse_object_tags_if_any(object_dict)
  return object_resource


def get_prefix_resource_from_xml_response(scheme, prefix_dict, bucket_name):
  """Creates resource_reference.PrefixResource from S3 API response.

  Args:
    scheme (storage_url.ProviderPrefix): Prefix used for provider URLs.
    prefix_dict (dict): The S3 API response representing a prefix.
    bucket_name (str): Bucket for the prefix.

  Returns:
    A resource_reference.PrefixResource instance.
  """
  prefix = prefix_dict['Prefix']
  return resource_reference.PrefixResource(
      storage_url.CloudUrl(
          scheme=scheme,
          bucket_name=bucket_name,
          resource_name=prefix),
      prefix=prefix)


def get_bucket_metadata_dict_from_request_config(request_config):
  """Returns S3 bucket metadata dict fields based on RequestConfig."""
  metadata = {}

  resource_args = request_config.resource_args
  if resource_args:
    if resource_args.cors_file_path is not None:
      metadata.update(
          xml_metadata_field_converters.process_cors(
              resource_args.cors_file_path))
    if resource_args.labels_file_path is not None:
      metadata.update(
          xml_metadata_field_converters.process_labels(
              resource_args.labels_file_path))
    if resource_args.lifecycle_file_path is not None:
      metadata.update(
          xml_metadata_field_converters.process_lifecycle(
              resource_args.lifecycle_file_path))
    if resource_args.location is not None:
      metadata['LocationConstraint'] = resource_args.location
    if resource_args.requester_pays is not None:
      metadata.update(
          xml_metadata_field_converters.process_requester_pays(
              resource_args.requester_pays))
    if resource_args.versioning is not None:
      metadata.update(
          xml_metadata_field_converters.process_versioning(
              resource_args.versioning))
    if (resource_args.web_error_page is not None or
        resource_args.web_main_page_suffix is not None):
      metadata.update(
          xml_metadata_field_converters.process_website(
              resource_args.web_error_page, resource_args.web_main_page_suffix))

  return metadata


def _process_value_or_clear_flag(metadata, key, value):
  """Sets appropriate metadata based on value."""
  if value == user_request_args_factory.CLEAR:
    metadata[key] = None
  elif value is not None:
    metadata[key] = value


def is_user_metadata_field_present_in_request_config(
    request_config,
    attributes_resource=None,
    known_posix=None,
):
  """Checks the presence of user_metadata fields in request_config."""
  resource_args = request_config.resource_args
  if resource_args is None:
    return False

  if request_config.predefined_acl_string is not None:
    return True

  for value in _S3_TO_GENERIC_FIELD_NAMES_.values():
    if getattr(resource_args, value, None):
      return True

  return metadata_util.has_updated_custom_fields(
      resource_args,
      request_config.preserve_posix,
      request_config.preserve_symlinks,
      attributes_resource=attributes_resource,
      known_posix=known_posix,
  )


def update_object_metadata_dict_from_request_config(
    object_metadata,
    request_config,
    attributes_resource=None,
    posix_to_set=None,
):
  """Returns S3 object metadata dict fields based on RequestConfig.

  Args:
    object_metadata (dict): Existing object metadata.
    request_config (request_config): May contain data to add to object_metadata.
    attributes_resource (Resource|None): If present, used for parsing POSIX and
      symlink data from a resource for the --preserve-posix and/or
      --preserve_symlink flags. This value is ignored unless it is an instance
      of FileObjectResource.
    posix_to_set (PosixAttributes|None): Set as custom metadata on target.

  """
  if request_config.predefined_acl_string is not None:
    object_metadata['ACL'] = translate_predefined_acl_string_to_s3(
        request_config.predefined_acl_string)

  resource_args = request_config.resource_args

  existing_metadata = object_metadata.get('Metadata', {})

  custom_fields_dict = metadata_util.get_updated_custom_fields(
      existing_metadata,
      request_config,
      attributes_resource=attributes_resource,
      known_posix=posix_to_set,
  )
  if custom_fields_dict is not None:
    object_metadata['Metadata'] = custom_fields_dict

  if resource_args:
    for field, value in _S3_TO_GENERIC_FIELD_NAMES_.items():
      _process_value_or_clear_flag(
          object_metadata,
          field,
          getattr(
              resource_args,
              value,
              None,
          ),
      )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`gcloud api-gateway api-configs delete` command."""


from googlecloudsdk.api_lib.api_gateway import api_configs
from googlecloudsdk.api_lib.api_gateway import operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.api_gateway import operations_util
from googlecloudsdk.command_lib.api_gateway import resource_args
from googlecloudsdk.core.console import console_io


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Delete(base.DeleteCommand):
  """Deletes a config from an API."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To delete an API Config 'my-config' in 'my-api', run:

          $ {command} my-config --api=my-api
        """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    base.ASYNC_FLAG.AddToParser(parser)
    resource_args.AddApiConfigResourceArg(parser, 'deleted', positional=True)

  def Run(self, args):
    """Run 'api-gateway api-configs delete'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The response from the Delete API call (or None if cancelled).
    """

    api_config_ref = args.CONCEPTS.api_config.Parse()

    # Prompt with a warning before continuing.
    console_io.PromptContinue(
        message='Are you sure? This will delete the API Config \'{}\', '
        'along with all of the associated consumer '
        'information.'.format(api_config_ref.RelativeName()),
        prompt_string='Continue anyway',
        default=True,
        throw_if_unattended=True,
        cancel_on_no=True)

    client = api_configs.ApiConfigClient()

    resp = client.Delete(api_config_ref)

    wait = 'Waiting for API Config [{}] to be deleted'.format(
        api_config_ref.Name())

    return operations_util.PrintOperationResult(
        resp.name, operations.OperationsClient(), wait_string=wait,
        is_async=args.async_)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to list authorizations."""

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class List(base.ListCommand):
  r"""List authorizations.

  List authorizations for a given connector, location and filter.

  ## EXAMPLES

  To list authorizations for a client user, run:

      $ {command} --connector=my-connector --location=us-central1 \
          --filter='client_user_id="my-user"' --project="my-project"
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument(
        '--connector',
        required=True,
        help='The connector ID to list authorizations for.',
    )
    parser.add_argument(
        '--location', required=True, help='Location of the connector.'
    )
    parser.display_info.AddFormat('table(name)')

  def Run(self, args):
    project = properties.VALUES.core.project.GetOrFail()

    # Parse the connector flag safely to support both short names and fully
    # qualified resource names.
    connector_ref = resources.REGISTRY.Parse(
        args.connector,
        params={'projectsId': project, 'locationsId': args.location},
        collection='iamconnectors.projects.locations.connectors',
    )

    parent = connector_ref.RelativeName()

    client = apis.GetClientInstance('iamconnectors', 'v1alpha')
    messages = client.MESSAGES_MODULE

    # Re-route standard gcloud filter to server-side filter to support the
    # required client_user_id filter on the server.
    server_filter = getattr(args, 'filter', None)

    # Clear args.filter to prevent the client-side framework from applying
    # local evaluation which breaks on client_user_id="str".
    args.filter = None

    request = messages.IamconnectorsProjectsLocationsConnectorsAuthorizationsListRequest(
        parent=parent,
        filter=server_filter,
        pageSize=args.page_size,
    )

    return list_pager.YieldFromList(
        client.projects_locations_connectors_authorizations,
        request,
        field='authorizations',
        limit=args.limit,
        batch_size_attribute='pageSize',
        batch_size=args.page_size,
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://networkconnectivity.googleapis.com/v1beta/'
DOCS_URL = 'https://cloud.google.com/network-connectivity/docs/reference/networkconnectivity/rest'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOBAL_HUBS = (
      'projects.locations.global.hubs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/global/hubs/{hubsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOBAL_HUBS_GROUPS = (
      'projects.locations.global.hubs.groups',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/global/hubs/{hubsId}/groups/'
              '{groupsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOBAL_HUBS_ROUTETABLES = (
      'projects.locations.global.hubs.routeTables',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/global/hubs/{hubsId}/'
              'routeTables/{routeTablesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOBAL_HUBS_ROUTETABLES_ROUTES = (
      'projects.locations.global.hubs.routeTables.routes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/global/hubs/{hubsId}/'
              'routeTables/{routeTablesId}/routes/{routesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_GLOBAL_POLICYBASEDROUTES = (
      'projects.locations.global.policyBasedRoutes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/global/policyBasedRoutes/'
              '{policyBasedRoutesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MULTICLOUDDATATRANSFERCONFIGS = (
      'projects.locations.multicloudDataTransferConfigs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'multicloudDataTransferConfigs/'
              '{multicloudDataTransferConfigsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MULTICLOUDDATATRANSFERCONFIGS_DESTINATIONS = (
      'projects.locations.multicloudDataTransferConfigs.destinations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'multicloudDataTransferConfigs/'
              '{multicloudDataTransferConfigsId}/destinations/'
              '{destinationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MULTICLOUDDATATRANSFERSUPPORTEDSERVICES = (
      'projects.locations.multicloudDataTransferSupportedServices',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'multicloudDataTransferSupportedServices/'
              '{multicloudDataTransferSupportedServicesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_OPERATIONS = (
      'projects.locations.operations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/operations/'
              '{operationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_REGIONALENDPOINTS = (
      'projects.locations.regionalEndpoints',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'regionalEndpoints/{regionalEndpointsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_REMOTETRANSPORTPROFILES = (
      'projects.locations.remoteTransportProfiles',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'remoteTransportProfiles/{remoteTransportProfilesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SPOKES = (
      'projects.locations.spokes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/spokes/'
              '{spokesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SPOKES_GATEWAYADVERTISEDROUTES = (
      'projects.locations.spokes.gatewayAdvertisedRoutes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/spokes/'
              '{spokesId}/gatewayAdvertisedRoutes/{gatewayAdvertisedRoutesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TRANSPORTS = (
      'projects.locations.transports',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/transports/'
              '{transportsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set the IAM policy for a model."""


from googlecloudsdk.api_lib.ml_engine import models
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.ml_engine import endpoint_util
from googlecloudsdk.command_lib.ml_engine import flags
from googlecloudsdk.command_lib.ml_engine import models_util
from googlecloudsdk.command_lib.ml_engine import region_util


def _AddSetIamPolicyArgs(parser):
  flags.GetModelResourceArg(
      positional=True, required=True,
      verb='to set IAM policy for').AddToParser(parser)
  flags.GetRegionArg(include_global=True).AddToParser(parser)
  iam_util.AddArgForPolicyFile(parser)


def _Run(args):
  region = region_util.GetRegion(args)
  with endpoint_util.MlEndpointOverrides(region=region):
    return models_util.SetIamPolicy(models.ModelsClient(), args.model,
                                    args.policy_file)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class SetIamPolicyGA(base.Command):
  """Set the IAM policy for a model.

  Sets the IAM policy for the given model as defined in a JSON or YAML file.

  See https://cloud.google.com/iam/docs/managing-policies for details of
  the policy file format and contents.

  ## EXAMPLES
  The following command will read am IAM policy defined in a JSON file
  'policy.json' and set it for the model `my_model`:

    $ {command} my_model policy.json
  """

  @staticmethod
  def Args(parser):
    _AddSetIamPolicyArgs(parser)

  def Run(self, args):
    return _Run(args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class SetIamPolicyBeta(base.Command):
  """Set the IAM policy for a model.

  Sets the IAM policy for the given model as defined in a JSON or YAML file.

  See https://cloud.google.com/iam/docs/managing-policies for details of
  the policy file format and contents.

  ## EXAMPLES
  The following command will read am IAM policy defined in a JSON file
  'policy.json' and set it for the model `my_model`:

    $ {command} my_model policy.json
  """

  @staticmethod
  def Args(parser):
    _AddSetIamPolicyArgs(parser)

  def Run(self, args):
    return _Run(args)



# -*- coding: utf-8 -*- #
#
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common utility functions for Developer Connect Insights Configs Discover App Hub and Discover Projects."""

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.asset import client_util as asset_client_util
from googlecloudsdk.command_lib.developer_connect import name
from googlecloudsdk.core import log
from googlecloudsdk.core.util import regional

_CLOUD_RUN_REVISION_ASSET_TYPE = 'run.googleapis.com/Revision'
_GKE_POD_ASSET_TYPE = 'k8s.io/Pod'


def _get_property_values(properties, keys):
  """Extracts values for given keys from a list of properties.

  Args:
    properties: A list of property objects, each with 'key' and 'value'.
    keys: A list of keys to look for.

  Returns:
    A tuple of values corresponding to the keys. If a key is not found,
    the corresponding value in the tuple will be None.
  """
  prop_dict = {p.key: p.value for p in properties}
  return tuple(prop_dict.get(key) for key in keys)


def query_cais_for_gke_assets(gke_workload):
  """Queries CAIS for assets associated with the given GKE workload.

  Args:
    gke_workload: A GKE workload.

  Returns:
    The assets that are associated with the GKE workload.
  """
  partial_pod_uri, parent = construct_partial_pod_uri_and_get_parent(
      gke_workload
  )
  log.status.Print(
      f'Finding artifacts running in {gke_workload.resource_name()}...'
  )
  search_request = (
      asset_client_util.GetMessages().CloudassetSearchAllResourcesRequest(
          scope=parent,
          query=f'name:{partial_pod_uri}',
          assetTypes=[_GKE_POD_ASSET_TYPE],
          readMask='name,versioned_resources,create_time,state',
      )
  )
  assets = list_pager.YieldFromList(
      asset_client_util.GetClient().v1,
      search_request,
      method='SearchAllResources',
      field='results',
      batch_size_attribute='pageSize',
  )
  return list(assets)


def query_cais_for_gke_assets_in_project(project):
  """Queries CAIS for GKE assets in a given GCP project.

  Args:
    project: A GCP Project.

  Returns:
    The GKE assets that are associated with the GCP Project.
  """
  # DCI does not track system namespaces.
  system_deployment_namespaces = [
      'config-management-system',
      'gke-gmp-system',
      'gke-managed-checkpointing',
      'gke-managed-cim',
      'gke-managed-lustrecsi',
      'gke-managed-parallelstorecsi',
      'gke-managed-system',
      'gke-managed-volumepopulator',
      'gke-system',
      'gmp-system',
      'istio-system',
      'knative-serving',
      'kube-system',
  ]

  # Dynamically build the exclusion query string.
  # Result matches: "NOT name:/namespaces/ns1/ NOT name:/namespaces/ns2/ ..."
  namespace_exclusions = [
      f'NOT name:/namespaces/{ns}/' for ns in system_deployment_namespaces
  ]
  query_string = ' '.join(namespace_exclusions)

  log.status.Print(f'Finding GKE artifacts running in {project}...')
  search_request = (
      asset_client_util.GetMessages().CloudassetSearchAllResourcesRequest(
          scope=f'projects/{project}',
          query=query_string,
          assetTypes=[_GKE_POD_ASSET_TYPE],
          readMask='name,versioned_resources,create_time,state',
      )
  )
  assets = list_pager.YieldFromList(
      asset_client_util.GetClient().v1,
      search_request,
      method='SearchAllResources',
      field='results',
      batch_size_attribute='pageSize',
  )
  return list(assets)


def query_cais_for_cloud_run_services(cloud_run_service):
  """Queries CAIS for assets associated with the given Cloud Run service.

  Args:
    cloud_run_service: A Cloud Run service.

  Returns:
    The assets that are associated with the Cloud Run service.
  """
  parent_full_service_resource_name = cloud_run_service.resource_name()
  log.status.Print(
      f'Finding artifacts running in {parent_full_service_resource_name}...'
  )
  search_request = (
      asset_client_util.GetMessages().CloudassetSearchAllResourcesRequest(
          scope=f'projects/{cloud_run_service.project_id}',
          query=f'parentFullResourceName:{parent_full_service_resource_name}',
          assetTypes=[_CLOUD_RUN_REVISION_ASSET_TYPE],
          readMask='name,versioned_resources',
      )
  )
  assets = list_pager.YieldFromList(
      asset_client_util.GetClient().v1,
      search_request,
      method='SearchAllResources',
      field='results',
      batch_size_attribute='pageSize',
  )
  return list(assets)


def query_cais_for_cloud_run_services_in_project(project):
  """Queries CAIS for Cloud Run service assets associated with the given GCP Project.

  Args:
    project: A GCP Project.

  Returns:
    The Cloud Run assets that are associated with the GCP Project.
  """
  log.status.Print(f'Finding Cloud Run artifacts running in {project}...')
  search_request = (
      asset_client_util.GetMessages().CloudassetSearchAllResourcesRequest(
          scope=f'projects/{project}',
          assetTypes=[_CLOUD_RUN_REVISION_ASSET_TYPE],
          readMask='name,versioned_resources',
      )
  )
  assets = list_pager.YieldFromList(
      asset_client_util.GetClient().v1,
      search_request,
      method='SearchAllResources',
      field='results',
      batch_size_attribute='pageSize',
  )
  return list(assets)


def construct_partial_pod_uri_and_get_parent(gke_workload):
  """Constructs a partial pod URI from a GKE workload and returns the parent.

  Args:
    gke_workload: A GKE workload.

  Returns:
    A partial pod URI that can be used to query CAIS for pods.
    A parent that can be used to query CAIS for resources.
  """
  project_info = name.Project(gke_workload.gke_namespace.gke_cluster.project)
  location = gke_workload.gke_namespace.gke_cluster.location_id
  cluster_id = gke_workload.gke_namespace.gke_cluster.cluster_id
  namespace_id = gke_workload.gke_namespace.namespace_id
  deployment_id = gke_workload.deployment_id
  parent = f'projects/{project_info.project_id}'

  # Zonal clusters use 'zones' and regional clusters use 'locations' in CAIS.
  location_type = (
      'locations'
      if regional.LocationToRegion(location) == location
      else 'zones'
  )

  partial_pod_uri = (
      f'//container.googleapis.com/projects/{project_info.project_id}/'
      f'{location_type}/{location}/clusters/{cluster_id}/k8s/namespaces/'
      f'{namespace_id}/pods/{deployment_id}'
  )
  return partial_pod_uri, parent


def get_artifact_uris_from_gke_assets(assets):
  """Gets artifact URIs from assets.

  Args:
    assets: A list of assets.

  Returns:
    A list of artifact uris.
  """
  artifact_uris = []
  for asset in assets:
    for versioned_resource in asset.versionedResources:
      if versioned_resource.version != 'v1':
        continue
      (spec_value,) = _get_property_values(
          versioned_resource.resource.additionalProperties, ['spec']
      )
      if (
          not spec_value
          or not spec_value.object_value
          or not spec_value.object_value.properties
      ):
        continue

      (containers_value,) = _get_property_values(
          spec_value.object_value.properties, ['containers']
      )
      if (
          not containers_value
          or not containers_value.array_value
          or not containers_value.array_value.entries
      ):
        continue

      containers_array = containers_value.array_value.entries
      if (
          not containers_array
          or not containers_array[0].object_value
          or not containers_array[0].object_value.properties
      ):
        continue

      (image_value,) = _get_property_values(
          containers_array[0].object_value.properties, ['image']
      )
      if image_value and image_value.string_value:
        artifact_uris.append(image_value.string_value)
  return artifact_uris


def _is_cloud_run_revision_active(status_value):
  """Checks if a Cloud Run revision is active based on its status.

  An 'Active' condition with a 'True' status on a revision means it is
  deployed and serving traffic. We filter for active revisions to ensure
  we are discovering artifacts that are currently deployed and running,
  rather than old or inactive revisions.

  Args:
    status_value: The status property of a Cloud Run revision resource.

  Returns:
    True if the revision is active, False otherwise.
  """
  (conditions_value,) = _get_property_values(
      status_value.object_value.properties, ['conditions']
  )
  conditions = getattr(conditions_value, 'array_value', None)

  if not conditions:
    return False

  for condition_entry in conditions.entries:
    condition_type_value, condition_status_value = _get_property_values(
        condition_entry.object_value.properties, ['type', 'status']
    )

    condition_type = getattr(condition_type_value, 'string_value', None)
    condition_status = getattr(condition_status_value, 'string_value', None)

    if condition_type == 'Active' and condition_status == 'True':
      return True
  return False


def _get_artifact_uris_from_cloud_run_versioned_resource(versioned_resource):
  """Gets image URIs from a Cloud Run versioned resource.

  Args:
    versioned_resource: A Cloud Run versioned resource.

  Returns:
    A list of artifact URIs from the active containers.
  """
  artifact_uris = []
  if versioned_resource.version != 'v1':
    return []

  spec_value, status_value = _get_property_values(
      versioned_resource.resource.additionalProperties, ['spec', 'status']
  )

  if not status_value or not hasattr(status_value, 'object_value'):
    return []

  if not _is_cloud_run_revision_active(status_value):
    return []

  if spec_value and hasattr(spec_value, 'object_value'):
    (containers_value,) = _get_property_values(
        spec_value.object_value.properties, ['containers']
    )
    containers = getattr(containers_value, 'array_value', None)

    if containers:
      for container_entry in containers.entries:
        (image_value,) = _get_property_values(
            container_entry.object_value.properties, ['image']
        )
        image_uri = getattr(image_value, 'string_value', None)
        if image_uri:
          artifact_uris.append(image_uri)
  return artifact_uris


def get_artifact_uris_from_cloud_run_assets(assets):
  """Gets image URIs from Cloud Run assets that are active and v1.

  Args:
    assets: A list of Cloud Run assets.

  Returns:
      A list of artifact URIs from the active containers.
  """
  artifact_uris = []
  for asset in assets:
    for vr in asset.versionedResources:
      artifact_uris.extend(
          _get_artifact_uris_from_cloud_run_versioned_resource(vr)
      )

  return artifact_uris



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library for interacting with the Security Command Center Findings API."""


import json
import re

from typing import Dict, List
from apitools.base.py import encoding
from googlecloudsdk.api_lib.scc.iac_remediation import const
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.scc.iac_remediation import errors


def GetClient():
  """Returns the Security Command Center Findings API client."""
  return apis.GetClientInstance(const.FINDINGS_API_NAME,
                                const.FINDINGS_API_VERSION)


def GetMessages():
  """Returns the Security Command Center Findings API messages."""
  return apis.GetMessagesModule(const.FINDINGS_API_NAME,
                                const.FINDINGS_API_VERSION)


def ParseName(finding_name) -> str:
  """Parses the finding name to get the finding id.

  Args:
    finding_name: Canonical name of the finding.

  Returns:
    Finding id, if found else throws an error
  """
  pattern = r"projects/(\d+)/sources/(\d+)/locations/(\w+)/findings/(\w+)"
  match = re.search(pattern, finding_name)
  if match:
    # Finding id is the 4th group captured from the regex pattern
    return match.group(4)
  else:
    raise errors.InvalidFindingNameError(finding_name)


def MakeApiCall(finding_org_id, finding_name) -> str:
  """Makes an API call to the Security Command Center Findings API.

  Args:
    finding_org_id: Organization ID of the finding
    finding_name: Canonical name of the finding.

  Returns:
    JSON response from the API call.
  """
  client = GetClient()
  messages = GetMessages()
  finding_id = ParseName(finding_name)
  request = messages.SecuritycenterOrganizationsSourcesFindingsListRequest()
  request.filter = f"name : \"{finding_id}\" "
  request.parent = f"organizations/{finding_org_id}/sources/-"
  resp = client.organizations_sources_findings.List(request)
  json_resp = encoding.MessageToJson(resp)
  ValidateFinding(json_resp)
  return json_resp


def FetchIAMBinding(
    finding_json: json
    )-> Dict[str, Dict[str, List[str]]]:
  """Fetches the IAMBindings from the finding data.

  Args:
    finding_json: JSON response from the API call to fetch the finding.

  Returns:
    IAM binding details per member.
  """
  iam_bindings = {}
  for binding in finding_json["listFindingsResults"][0]["finding"][
      "iamBindings"
  ]:
    if binding["member"] not in iam_bindings:
      iam_bindings[binding["member"]] = dict()
    if binding["action"] == "ADD":
      if "ADD" not in iam_bindings[binding["member"]]:
        iam_bindings[binding["member"]]["ADD"] = []
      if binding["role"] not in iam_bindings[binding["member"]]["ADD"]:
        iam_bindings[binding["member"]]["ADD"].append(binding["role"])
    elif binding["action"] == "REMOVE":
      if "REMOVE" not in iam_bindings[binding["member"]]:
        iam_bindings[binding["member"]]["REMOVE"] = []
      if binding["role"] not in iam_bindings[binding["member"]]["REMOVE"]:
        iam_bindings[binding["member"]]["REMOVE"].append(binding["role"])
  return iam_bindings


def FetchResourceName(finding_json: json) -> str:
  """Fetches the resource name from the finding data.

  Args:
    finding_json: JSON response from the API call to fetch the finding.

  Returns:
    Resource name for which the finding was generated.
  """
  return finding_json["listFindingsResults"][0]["resource"]["displayName"]


def ValidateFinding(finding_data):
  """Checks if the finding is supported or not.

  Args:
    finding_data: JSON response from the API call.
  """
  try:
    finding_data = json.loads(finding_data)
    finding = finding_data["listFindingsResults"][0]["finding"]
  except:
    raise errors.FindingNotFoundError()
  finding_category = finding["category"]
  if finding_category not in const.SUPPORTED_FINDING_CATEGORIES:
    raise errors.UnsupportedFindingCategoryError(finding_category)
  iam_bindings = FetchIAMBinding(finding_data)
  if (
      len(iam_bindings) > const.SUPPORTED_IAM_MEMBER_COUNT_LIMIT
      or len(iam_bindings) < 1
  ):
    raise errors.ExcessiveMembersError(len(iam_bindings))



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating organization security policy associations."""


import sys

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.org_security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.org_security_policies import flags
from googlecloudsdk.command_lib.compute.org_security_policies import org_security_policies_utils
from googlecloudsdk.core import log
import six


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Create(base.CreateCommand):
  """Create a new association between a security policy and an organization or folder resource.

  *{command}* is used to create organization security policy associations. An
  organization security policy is a set of rules that controls access to various
  resources.

  This command has billing implications. Projects in the hierarchy with
  effective hierarchical security policies will be automatically enrolled into
  Cloud Armor Enterprise if not already enrolled.
  """

  @classmethod
  def Args(cls, parser):
    flags.AddArgsCreateAssociation(parser)
    parser.display_info.AddCacheUpdater(flags.OrgSecurityPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    org_security_policy = client.OrgSecurityPolicy(
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())

    name = None
    attachment_id = None
    replace_existing_association = False
    excluded_projects = []
    excluded_folders = []

    if args.IsSpecified('name'):
      name = args.name

    if args.IsSpecified('project_number'):
      attachment_id = 'projects/' + args.project_number
      if name is None:
        name = 'project-' + args.project_number

    if args.IsSpecified('folder'):
      attachment_id = 'folders/' + args.folder
      if name is None:
        name = 'folder-' + args.folder

    if args.IsSpecified('organization') and attachment_id is None:
      attachment_id = 'organizations/' + args.organization
      if name is None:
        name = 'organization-' + args.organization

    if attachment_id is None:
      log.error(
          'Must specify attachment ID with --organization=ORGANIZATION or '
          '--folder=FOLDER or --project-number=PROJECT.')
      sys.exit()

    if args.IsSpecified('excluded_projects'):
      excluded_projects = args.excluded_projects

    if args.IsSpecified('excluded_folders'):
      excluded_folders = args.excluded_folders

    if args.replace_association_on_target:
      replace_existing_association = True

    association = holder.client.messages.SecurityPolicyAssociation(
        attachmentId=attachment_id,
        name=name,
        excludedProjects=excluded_projects,
        excludedFolders=excluded_folders,
    )

    log.status.Print("""\
  This command has billing implications. Projects in the hierarchy with
  effective organization security policies will be automatically enrolled into
  Cloud Armor Enterprise if not already enrolled.""")
    security_policy_id = org_security_policies_utils.GetSecurityPolicyId(
        org_security_policy,
        args.security_policy,
        organization=args.organization)
    return org_security_policy.AddAssociation(
        association=association,
        security_policy_id=security_policy_id,
        replace_existing_association=replace_existing_association,
        only_generate_request=False)


Create.detailed_help = {
    'EXAMPLES':
        """\
    To associate an organization security policy under folder with ID
    ``123456789'' to folder ``987654321'', run:

      $ {command} --security-policy=123456789 --folder=987654321
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common utilities for the Developer Connect API."""

import re

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
import six.moves.urllib.parse


API_NAME = 'developerconnect'
API_VERSION_1 = 'v1'


def GetApiVersion(release_track):
  if release_track == base.ReleaseTrack.GA:
    return API_VERSION_1
  else:
    return None


def GetApiServiceName(api_version):
  """Gets the service name based on the configured API endpoint."""
  endpoint = apis.GetEffectiveApiEndpoint(API_NAME, api_version)
  hostname = six.moves.urllib.parse.urlparse(endpoint).hostname
  if hostname:
    # Remove mTLS markers.
    hostname = hostname.replace('.mtls.', '.')
    # Remove regional REP (Regional Endpoint) markers.
    hostname = re.sub(r'\.[a-z0-9-]+\.(?:p\.)?rep\.', '.', hostname)
  return hostname



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a new AlloyDB cluster."""


import random

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.api_lib.alloydb import cluster_operations
from googlecloudsdk.api_lib.backupdr.backup_plan_associations import BackupPlanAssociationsClient
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.alloydb import cluster_helper
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


# TODO: b/312466999 - Change @base.DefaultUniverseOnly to
# @base.UniverseCompatible once b/312466999 is fixed.
# See go/gcloud-cli-running-tpc-tests.
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a new AlloyDB cluster within a given project."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To create a new cluster, run:

          $ {command} my-cluster --region=us-central1 --password=postgres
        """,
  }

  @classmethod
  def Args(cls, parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    alloydb_messages = api_util.GetMessagesModule(cls.ReleaseTrack())
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddRegion(parser)
    flags.AddCluster(parser)
    flags.AddNetwork(parser)
    flags.AddPassword(parser)
    flags.AddAllocatedIPRangeName(parser)
    kms_resource_args.AddKmsKeyResourceArg(
        parser,
        'cluster',
        permission_info=(
            "The 'AlloyDB Service Agent' service account must hold permission"
            " 'Cloud KMS CryptoKey Encrypter/Decrypter'"
        ),
    )
    flags.AddAutomatedBackupFlags(
        parser, alloydb_messages, cls.ReleaseTrack(), update=False
    )
    flags.AddContinuousBackupConfigFlags(parser, cls.ReleaseTrack())
    flags.AddDatabaseVersion(parser, alloydb_messages)
    flags.AddEnablePrivateServiceConnect(parser)
    flags.AddMaintenanceWindow(parser, alloydb_messages)
    flags.AddDenyMaintenancePeriod(parser, alloydb_messages)
    flags.AddSubscriptionType(parser, alloydb_messages)
    flags.AddTags(parser)
    flags.AddDataplexIntegrationFlags(parser)

  def ConstructCreateRequestFromArgs(self, alloydb_messages, location_ref,
                                     args):
    return cluster_helper.ConstructCreateRequestFromArgsGA(
        alloydb_messages, location_ref, args)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    if getattr(args, 'backupdr_backup_plan', None) and args.async_:
      raise exceptions.InvalidArgumentException(
          '--backupdr-backup-plan',
          'Cannot use --async with --backupdr-backup-plan.')
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    location_ref = client.resource_parser.Create(
        'alloydb.projects.locations',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region)
    req = self.ConstructCreateRequestFromArgs(alloydb_messages, location_ref,
                                              args)
    op = alloydb_client.projects_locations_clusters.Create(req)
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='alloydb.projects.locations.operations')
    log.status.Print('Operation ID: {}'.format(op_ref.Name()))
    if getattr(args, 'backupdr_backup_plan', None):
      log.status.Print(
          'Waiting for cluster creation to complete before associating BackupDR'
          ' plan ...'
      )
      cluster_operations.Await(op_ref, 'Creating cluster', self.ReleaseTrack())
      cluster_ref = client.resource_parser.Create(
          'alloydb.projects.locations.clusters',
          projectsId=properties.VALUES.core.project.GetOrFail,
          locationsId=args.region,
          clustersId=args.cluster
      )
      cluster_uri = cluster_ref.RelativeName()

      bpa_client = BackupPlanAssociationsClient()

      backup_plan_ref = client.resource_parser.Parse(
          args.backupdr_backup_plan,
          collection='backupdr.projects.locations.backupPlans',
      )
      backup_plan_location = backup_plan_ref.Parent().Name()

      # Constructing BPA name. BPA is created in same project as cluster, but
      # in the same region as backup plan.
      # BPA ID is derived from the cluster ID.
      bpa_id = f'{args.cluster}-bpa-{random.randint(0, 100_000)}'
      bpa_ref = client.resource_parser.Create(
          'backupdr.projects.locations.backupPlanAssociations',
          projectsId=properties.VALUES.core.project.GetOrFail,
          locationsId=backup_plan_location,
          backupPlanAssociationsId=bpa_id
      )

      resource_type = 'alloydb.googleapis.com/Cluster'

      log.status.Print(
          f'Creating backup plan association {bpa_id} for cluster'
          f' {args.cluster} with plan {args.backupdr_backup_plan} ...'
      )
      try:
        bpa_op = bpa_client.Create(
            bpa_ref, backup_plan_ref, cluster_uri, resource_type
        )
        bpa_client.WaitForOperation(
            operation_ref=bpa_client.GetOperationRef(bpa_op),
            message=(
                'Creating backup plan association'
                f' [{bpa_ref.RelativeName()}] '
            ),
        )
        log.status.Print(
            f'\nSuccessfully associated BackupDR plan'
            f' {args.backupdr_backup_plan} with cluster {args.cluster}'
        )
      except apitools_exceptions.HttpError as e:
        error_message = (
            f'Failed to create Backup Plan Association {bpa_id}: {repr(e)}'
        )
        log.warning(error_message)
        retry_command = (
            'gcloud alpha backup-dr backup-plan-associations create'
            f' {bpa_id} \\ \n'
            f'\t --project={properties.VALUES.core.project.GetOrFail()} \\ \n'
            f'\t --location={args.region} \\ \n'
            f'\t --backup-plan={args.backupdr_backup_plan} \\ \n'
            f'\t --resource={cluster_uri} \\ \n'
            f'\t --resource-type={resource_type}'
        )
        log.info(f'To retry, you can run: \n  {retry_command}')

    elif not args.async_:
      cluster_operations.Await(op_ref, 'Creating cluster', self.ReleaseTrack())
    return op


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a new AlloyDB cluster within a given project."""

  @classmethod
  def Args(cls, parser):
    super(CreateBeta, cls).Args(parser)

  def ConstructCreateRequestFromArgs(
      self, alloydb_messages, location_ref, args
  ):
    return cluster_helper.ConstructCreateRequestFromArgsBeta(
        alloydb_messages, location_ref, args
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(CreateBeta):
  """Create a new AlloyDB cluster within a given project."""

  @classmethod
  def Args(cls, parser):
    super(CreateAlpha, cls).Args(parser)
    flags.AddBackupDRBackupPlan(parser)

  def ConstructCreateRequestFromArgs(
      self, alloydb_messages, location_ref, args
  ):
    return cluster_helper.ConstructCreateRequestFromArgsAlpha(
        alloydb_messages, location_ref, args
    )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Support library for the auth command."""

import json
import os
import textwrap

from google.auth import credentials as google_auth_creds
from google.auth import environment_vars as auth_env_vars
from google.auth import jwt
from googlecloudsdk.api_lib.auth import exceptions as auth_exceptions
from googlecloudsdk.api_lib.auth import util as api_auth_util
from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
from googlecloudsdk.api_lib.iamcredentials import util as impersonation_util
from googlecloudsdk.calliope import exceptions as c_exc
from googlecloudsdk.command_lib.projects import util as project_util
from googlecloudsdk.core import config
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.credentials import creds as c_creds
from googlecloudsdk.core.credentials import store as c_store
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import platforms


SERVICEUSAGE_PERMISSION = 'serviceusage.services.use'


def GetTrustedScopesWithDrive():
  """Returns the list of trusted scopes including Google Drive."""
  # bq needs this as part of its auth flow to support the Drive scope.
  return list(config.CLOUDSDK_SCOPES) + [api_auth_util.GOOGLE_DRIVE_SCOPE]


def LoadCredentialsWithScopes(
    account,
    scopes,
    trusted_scopes,
    impersonation_lifetime=None,
    force_refresh=False,
    min_expiry=None,
):
  """Loads credentials with specified scopes and lifetime.

  Args:
    account: str, The account to get the access token for.
    scopes: [str], The scopes to authorize for.
    trusted_scopes: [str], The list of trusted scopes to validate against.
    impersonation_lifetime: int, Access token lifetime for impersonation.
    force_refresh: bool, Whether to force a refresh of the credentials.
    min_expiry: str, Duration representing the amount of time between now and
      token_expiry_time to compare against for refresh.

  Returns:
    The loaded credentials.
  """
  # Do not auto cache the custom scoped access token. Otherwise, it'll affect
  # other gcloud CLIs that depends on cloud-platform scopes.
  cache_only_rapt = True if scopes else False

  cred = c_store.Load(
      account,
      allow_account_impersonation=True,
      cache_only_rapt=cache_only_rapt,
  )

  # c_store.Load already refreshed the cred, so we don't need to refresh the
  # cred unless we need to alter the cred in the code below, for example,
  # changing the scopes.
  should_refresh_again = False

  if scopes:
    # refresh again due to altered scopes
    should_refresh_again = True
    cred_type = c_creds.CredentialTypeGoogleAuth.FromCredentials(cred)
    if cred_type not in [
        c_creds.CredentialTypeGoogleAuth.USER_ACCOUNT,
        c_creds.CredentialTypeGoogleAuth.SERVICE_ACCOUNT,
    ]:
      # TODO(b/223649175): Add support for other credential types(e.g GCE).
      log.warning(
          '`--scopes` flag may not work as expected and will be ignored '
          'for account type {}.'.format(cred_type.key)
      )
    full_scopes = scopes + [
        api_auth_util.OPENID,
        api_auth_util.USER_EMAIL_SCOPE,
    ]

    # non user account credential types
    if isinstance(cred, google_auth_creds.Scoped):
      cred = cred.with_scopes(full_scopes)
    else:
      requested_scopes = set(scopes)
      trusted_scopes_set = set(trusted_scopes)
      if not requested_scopes.issubset(trusted_scopes_set):
        raise c_exc.InvalidArgumentException(
            '--scopes',
            'Invalid scopes value. Please make sure the scopes are from [{0}]'
            .format(trusted_scopes),
        )
      # pylint:disable=protected-access
      cred._scopes = full_scopes

  if c_creds.IsImpersonatedAccountCredentials(cred) and impersonation_lifetime:
    # refresh again due to altered lifetime
    should_refresh_again = True
    cred._lifetime = impersonation_lifetime  # pylint: disable=protected-access

  if force_refresh or should_refresh_again:
    c_store.Refresh(cred)
  elif min_expiry is not None:
    c_store.RefreshIfExpireWithinWindow(cred, min_expiry)

  return cred


class MissingPermissionOnQuotaProjectError(c_creds.ADCError):
  """An error when ADC does not have permission to bill a quota project."""


class AddQuotaProjectError(c_creds.ADCError):
  """An error when quota project ID is added to creds that don't support it."""


def IsGceAccountCredentials(cred):
  """Checks if the credential is a Compute Engine service account credential."""
  # Import only when necessary to decrease the startup time.
  # pylint: disable=g-import-not-at-top
  import google.auth.compute_engine as google_auth_gce

  return isinstance(cred, google_auth_gce.credentials.Credentials)


def IsServiceAccountCredential(cred):
  """Checks if the credential is a service account credential."""
  # Import only when necessary to decrease the startup time.
  # pylint: disable=g-import-not-at-top
  import google.oauth2.service_account as google_auth_service_account

  return isinstance(cred, google_auth_service_account.Credentials)


def IsImpersonationCredential(cred):
  """Checks if the credential is an impersonated service account credential."""
  return impersonation_util.ImpersonationAccessTokenProvider.IsImpersonationCredential(
      cred
  )


def ValidIdTokenCredential(cred):
  return (
      IsImpersonationCredential(cred)
      or IsServiceAccountCredential(cred)
      or IsGceAccountCredentials(cred)
  )


def PromptIfADCEnvVarIsSet():
  """Warns users if ADC environment variable is set."""
  override_file = config.ADCEnvVariable()
  if override_file:
    message = textwrap.dedent(
        """
          The environment variable [{envvar}] is set to:
            [{override_file}]
          Credentials will still be generated to the default location:
            [{default_file}]
          To use these credentials, unset this environment variable before
          running your application.
          """.format(
            envvar=auth_env_vars.CREDENTIALS,
            override_file=override_file,
            default_file=config.ADCFilePath(),
        )
    )
    console_io.PromptContinue(
        message=message, throw_if_unattended=True, cancel_on_no=True
    )


def WriteGcloudCredentialsToADC(creds, add_quota_project=False):
  """Writes gclouds's credential from auth login to ADC json."""
  # TODO(b/190114370): We will also support writing service account creds.
  if not c_creds.IsUserAccountCredentials(
      creds
  ) and not c_creds.IsExternalAccountCredentials(creds):
    log.warning(
        'Credentials cannot be written to application default '
        'credentials because it is not a user or external account '
        'credential.'
    )
    return
  # Quota project ID should not be added to non-user credentials.
  if c_creds.IsExternalAccountCredentials(creds) and add_quota_project:
    raise AddQuotaProjectError(
        'The application default credentials are external account credentials, '
        'quota project cannot be added.'
    )

  PromptIfADCEnvVarIsSet()
  if add_quota_project:
    DumpADCOptionalQuotaProject(creds)
  else:
    c_creds.ADC(creds).DumpADCToFile()


def GetADCAsJson():
  """Reads ADC from disk and converts it to a json object."""
  if not os.path.isfile(config.ADCFilePath()):
    return None
  with files.FileReader(config.ADCFilePath()) as f:
    return json.load(f)


def GetQuotaProjectFromADC():
  """Reads the quota project ID from ADC json file and return it."""
  adc_json = GetADCAsJson()
  try:
    return adc_json['quota_project_id']
  except (KeyError, TypeError):
    return None


def AssertADCExists():
  adc_path = config.ADCFilePath()
  if not os.path.isfile(adc_path):
    raise c_exc.BadFileException(
        'Application default credentials have not been set up. '
        'Run $ gcloud auth application-default login to set it up first.'
    )


def ADCIsUserAccount():
  """Returns whether the ADC credentials correspond to a user account or not."""
  cred_file = config.ADCFilePath()
  creds, _ = c_creds.GetGoogleAuthDefault().load_credentials_from_file(
      cred_file
  )
  return c_creds.IsUserAccountCredentials(
      creds
  ) or c_creds.IsExternalAccountUserCredentials(creds)


def AdcHasGivenPermissionOnProject(project_id, permissions):
  AssertADCExists()
  project_ref = project_util.ParseProject(project_id)
  return _AdcHasGivenPermissionOnProjectHelper(project_ref, permissions)


def _AdcHasGivenPermissionOnProjectHelper(project_ref, permissions):
  cred_file_override_old = properties.VALUES.auth.credential_file_override.Get()
  try:
    properties.VALUES.auth.credential_file_override.Set(config.ADCFilePath())
    granted_permissions = projects_api.TestIamPermissions(
        project_ref, permissions
    ).permissions
    return set(permissions) == set(granted_permissions)
  finally:
    properties.VALUES.auth.credential_file_override.Set(cred_file_override_old)


# Create this function to ease unit test mocks.
def GetAdcRealPath(adc_path):
  return os.path.realpath(adc_path)


def LogADCIsWritten(adc_path):
  """Prints the confirmation when ADC file was successfully written."""
  real_path = adc_path
  if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
    real_path = GetAdcRealPath(adc_path)
  log.status.Print('\nCredentials saved to file: [{}]'.format(real_path))
  log.status.Print(
      '\nThese credentials will be used by any library that requests '
      'Application Default Credentials (ADC).'
  )
  # See https://bugs.python.org/issue40377 for the issue with python
  # from Microsoft store. The ADC file is transparently redirected to a
  # different path which client libraries do not expect, so cannot locate.
  if real_path != adc_path:
    log.warning(
        'You may be running gcloud with a python interpreter installed '
        'from Microsoft Store which is not supported by this command. '
        'Run `gcloud topic startup` for instructions to select a '
        'different python interpreter. Otherwise, you have to '
        'set the environment variable `GOOGLE_APPLICATION_CREDENTIALS` '
        'to the file path `{}`. See '
        'https://cloud.google.com/docs/authentication/'
        'getting-started#setting_the_environment_variable '
        'for more information.'.format(real_path)
    )


def LogQuotaProjectAdded(quota_project):
  log.status.Print(
      '\nQuota project "{}" was added to ADC which can be used by Google '
      'client libraries for billing and quota. Note that some services may '
      'still bill the project owning the resource.'.format(quota_project)
  )


def LogQuotaProjectNotFound():
  log.warning(
      '\nCannot find a quota project to add to ADC. You might receive '
      'a "quota exceeded" or "API not enabled" error. Run $ gcloud '
      'auth application-default set-quota-project to add '
      'a quota project.'
  )


def LogMissingPermissionOnQuotaProject(quota_project):
  log.warning(
      '\nCannot add the project "{}" to ADC as the quota project because the '
      'account in ADC does not have the "{}" permission on this project. '
      'You might receive a "quota_exceeded" or "API not enabled" error. '
      'Run $ gcloud auth application-default set-quota-project to add a quota '
      'project.'.format(quota_project, SERVICEUSAGE_PERMISSION)
  )


def LogQuotaProjectDisabled():
  log.warning(
      '\nQuota project is disabled. You might receive a "quota exceeded" or '
      '"API not enabled" error. Run $ gcloud auth application-default '
      'set-quota-project to add a quota project.'
  )


def DumpADC(credentials, quota_project_disabled=False):
  """Dumps the given credentials to ADC file.

  Args:
     credentials: a google-auth credential, the credentials to dump.
     quota_project_disabled: bool, If quota project is explicitly disabled by
       users using flags.
  """
  adc_path = c_creds.ADC(credentials).DumpADCToFile()
  LogADCIsWritten(adc_path)
  if quota_project_disabled:
    LogQuotaProjectDisabled()


def DumpADCOptionalQuotaProject(credentials):
  """Dumps the given credentials to ADC file with an optional quota project.

  Loads quota project from gcloud's context and writes it to application default
  credentials file if the credentials has the "serviceusage.services.use"
  permission on the quota project..

  Args:
     credentials: a google-auth credentials, the credentials to dump.
  """
  adc_path = c_creds.ADC(credentials).DumpADCToFile()
  LogADCIsWritten(adc_path)

  quota_project = c_creds.GetQuotaProject(
      credentials, force_resource_quota=True
  )
  if not quota_project:
    LogQuotaProjectNotFound()
  elif AdcHasGivenPermissionOnProject(
      quota_project, permissions=[SERVICEUSAGE_PERMISSION]
  ):
    c_creds.ADC(credentials).DumpExtendedADCToFile(quota_project=quota_project)
    LogQuotaProjectAdded(quota_project)
  else:
    LogMissingPermissionOnQuotaProject(quota_project)


def AddQuotaProjectToADC(quota_project):
  """Adds the quota project to the existing ADC file.

  Quota project is only added to ADC when the credentials have the
  "serviceusage.services.use" permission on the project.

  Args:
    quota_project: str, The project id of a valid GCP project to add to ADC.

  Raises:
    MissingPermissionOnQuotaProjectError: If the credentials do not have the
      "serviceusage.services.use" permission.
  """
  AssertADCExists()
  if not ADCIsUserAccount():
    raise c_exc.BadFileException(
        'The application default credentials are not user credentials, quota '
        'project cannot be added.'
    )

  credentials, _ = c_creds.GetGoogleAuthDefault().load_credentials_from_file(
      config.ADCFilePath()
  )

  previous_quota_project = credentials.quota_project_id

  # we call the cloudresourcemanager api under the quota of gcloud's quota
  # project to check if the user is granted the usage permission. Given the
  # current project can have this api disabled. So removing the quota project
  # from ADC so that check can be done regardless if cloudresourcemanager api
  # is enabled or disabled on the project.
  c_creds.ADC(credentials).DumpADCToFile()

  if not AdcHasGivenPermissionOnProject(
      quota_project, permissions=[SERVICEUSAGE_PERMISSION]
  ):
    c_creds.ADC(credentials).DumpExtendedADCToFile(
        quota_project=previous_quota_project
    )

    raise MissingPermissionOnQuotaProjectError(
        'Cannot add the project "{}" to application default credentials (ADC) '
        'as a quota project because the account in ADC does not have the '
        '"{}" permission on this project.'.format(
            quota_project, SERVICEUSAGE_PERMISSION
        )
    )

  adc_path = c_creds.ADC(credentials).DumpExtendedADCToFile(
      quota_project=quota_project
  )

  LogADCIsWritten(adc_path)
  LogQuotaProjectAdded(quota_project)


def DumpImpersonatedServiceAccountToADC(
    credentials, target_principal, delegates, scopes=None
):
  adc_path = c_creds.ADC(
      credentials, target_principal, delegates, scopes
  ).DumpADCToFile()
  LogADCIsWritten(adc_path)


def ExtractAndValidateAccount(account, creds):
  """Extracts account from creds and validates it against account."""
  decoded_id_token = jwt.decode(creds.id_token, verify=False)
  web_flow_account = decoded_id_token['email']
  if account and account.lower() != web_flow_account.lower():
    raise auth_exceptions.WrongAccountError(
        'You attempted to log in as account [{account}] but the received '
        'credentials were for account [{web_flow_account}].\n\n'
        'Please check that your browser is logged in as account [{account}] '
        'and that you are using the correct browser profile.'.format(
            account=account, web_flow_account=web_flow_account
        )
    )
  return web_flow_account



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a user in a given cluster.

Creates a user in a given cluster with specified username, type, and password.
"""




from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.command_lib.alloydb import user_helper
from googlecloudsdk.core import properties


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Creates a user in a given cluster.

  Creates a user in a given cluster with specified username, cluster, region,
  type, and password.
  """

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
      To create a new user, run:

        $ {command} my-username --cluster=my-cluster --region=us-central1 --password=postgres
      """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs
    """
    flags.AddUsername(parser)
    flags.AddCluster(parser, False)
    flags.AddRegion(parser)
    flags.AddUserPassword(parser)
    flags.AddUserType(parser)
    flags.AddCreateSuperuser(parser)
    flags.AddDBRoles(parser)
    flags.AddKeepExtraRoles(parser)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    cluster_ref = client.resource_parser.Create(
        'alloydb.projects.locations.clusters',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region,
        clustersId=args.cluster,
    )
    req = user_helper.ConstructCreateRequestFromArgs(
        client, alloydb_messages, cluster_ref, args
    )
    return alloydb_client.projects_locations_clusters_users.Create(req)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Profiles-specific printer and functions for generating CSV formats."""


import csv
import sys

from googlecloudsdk.core.resource import custom_printer_base as cp


PROFILES_PRINTER_FORMAT = "csvprofile"


def amount_to_decimal(cost):
  """Converts cost to a decimal representation."""
  units = cost.units
  if not units:
    units = 0
  decimal_value = +(units + cost.nanos / 1e9)
  return f"{decimal_value:.3f}"


def get_decimal_cost(costs):
  """Returns the cost per million normalized output tokens as a decimal.

  Args:
    costs: The costs to convert.
  """
  output_token_cost = "N/A"
  if costs and costs[0].costPerMillionOutputTokens:
    output_token_cost = amount_to_decimal(
        costs[0].costPerMillionOutputTokens
    )
  input_token_cost = "N/A"
  if costs and costs[0].costPerMillionInputTokens:
    input_token_cost = amount_to_decimal(costs[0].costPerMillionInputTokens)
  return (input_token_cost, output_token_cost)


def _transform_profiles(profiles):
  """Transforms profiles to a CSV format, including cost conversions."""
  csv_data = []
  header = [
      "Instance Type",
      "Accelerator Type",
      "Model Name",
      "Model Server Name",
      "Model Server Version",
      "Output Tokens/s",
      "NTPOT (ms)",
      "TTFT (ms)",
      "ITL (ms)",
      "QPS",
      "Cost/M Input Tokens",
      "Cost/M Output Tokens",
      "Use Case",
      "Average Input Length",
      "Average Output Length",
      "Serving Stack",
      "Serving Stack Version",
  ]
  csv_data.append(header)
  for profile in profiles:
    if profile.performanceStats:
      for stats in profile.performanceStats:
        input_token_cost, output_token_cost = get_decimal_cost(
            stats.cost
        )
        row = [
            profile.instanceType,
            profile.acceleratorType,
            profile.modelServerInfo.model,
            profile.modelServerInfo.modelServer,
            profile.modelServerInfo.modelServerVersion,
            stats.outputTokensPerSecond,
            stats.ntpotMilliseconds,
            stats.ttftMilliseconds,
            getattr(stats, "itlMilliseconds", ""),
            stats.queriesPerSecond,
            input_token_cost,
            output_token_cost,
            profile.workloadSpec.useCase,
            profile.workloadSpec.averageInputLength,
            profile.workloadSpec.averageOutputLength,
            profile.servingStack.name if profile.servingStack else None,
            profile.servingStack.version if profile.servingStack else None,
        ]
        csv_data.append(row)
  return csv_data


class ProfileCSVPrinter(cp.CustomPrinterBase):
  """Prints a service's profile in a custom human-readable format."""

  def Transform(self, profiles):
    """Transforms a List[TrafficTargetPair] into a CSV format."""
    return _transform_profiles(profiles)

  def Print(self, resources, single=True, intermediate=False):
    """Overrides ResourcePrinter.Print to set single=True."""
    writer = csv.writer(sys.stdout, lineterminator="\n")
    writer.writerows(self.Transform(resources))



"""Generated message classes for cloudtrace version v2beta1.

Sends application trace data to Cloud Trace for viewing. Trace data is
collected for all App Engine applications by default. Trace data from other
applications can be provided using this API. This library is used to interact
with the Cloud Trace API directly. If you are looking to instrument your
application for Cloud Trace, we recommend using OpenTelemetry.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'cloudtrace'


class BigQueryOutputConfig(_messages.Message):
  r"""Configuration for the output that is specific to BigQuery when choosing
  a BigQuery dataset as the output destination.

  Enums:
    VariantValueValuesEnum: Optional. Schema variant that should be used when
      exporting data to BigQuery.

  Fields:
    variant: Optional. Schema variant that should be used when exporting data
      to BigQuery.
  """

  class VariantValueValuesEnum(_messages.Enum):
    r"""Optional. Schema variant that should be used when exporting data to
    BigQuery.

    Values:
      SCHEMA_VARIANT_UNSPECIFIED: (Input only) sentinel indicating that the
        API should choose automatically. On create, a value will be selected
        and persisted. Subsequent reads from the API will return a non-default
        value indicating what was selected.
      FIELDS_EXPANDED_AS_COLUMNS: Export data with all attributes and label
        keys of the span expanded as columns dynamically, and each span as a
        row. The attribute/label columns are assigned their associated values
        in that row. We recommend this mode of operation when the number of
        unique label keys are relatively small and stable, as it provides a
        simpler, more convenient query experience.
      CONDENSED_STATIC_JSON: Export data as condensed, json-formatted columns.
        Choose this mode when unique label keys have high cardinality and/or
        change frequently. This schema variant is more scalable and lacks the
        potential issue of reaching the maximum column limit in BigQuery. At
        the same time, this variant is more difficult to query compared with
        `FIELDS_EXPANDED_AS_COLUMNS` mode, often requiring the use of BigQuery
        JSON functions
        (https://cloud.google.com/bigquery/docs/reference/standard-
        sql/json_functions) in order to extract particular labels and use them
        in queries.
    """
    SCHEMA_VARIANT_UNSPECIFIED = 0
    FIELDS_EXPANDED_AS_COLUMNS = 1
    CONDENSED_STATIC_JSON = 2

  variant = _messages.EnumField('VariantValueValuesEnum', 1)


class CloudtraceProjectsTraceSinksCreateRequest(_messages.Message):
  r"""A CloudtraceProjectsTraceSinksCreateRequest object.

  Fields:
    parent: Required. The resource in which to create the sink (currently only
      project sinks are supported): "projects/[PROJECT_ID]" Examples:
      `"projects/my-trace-project"`, `"projects/123456789"`.
    traceSink: A TraceSink resource to be passed as the request body.
  """

  parent = _messages.StringField(1, required=True)
  traceSink = _messages.MessageField('TraceSink', 2)


class CloudtraceProjectsTraceSinksDeleteRequest(_messages.Message):
  r"""A CloudtraceProjectsTraceSinksDeleteRequest object.

  Fields:
    name: Required. The full resource name of the sink to delete, including
      the parent resource and the sink identifier:
      "projects/[PROJECT_NUMBER]/traceSinks/[SINK_ID]" Example:
      `"projects/12345/traceSinks/my-sink-id"`.
  """

  name = _messages.StringField(1, required=True)


class CloudtraceProjectsTraceSinksGetRequest(_messages.Message):
  r"""A CloudtraceProjectsTraceSinksGetRequest object.

  Fields:
    name: Required. The resource name of the sink:
      "projects/[PROJECT_NUMBER]/traceSinks/[SINK_ID]" Example:
      `"projects/12345/traceSinks/my-sink-id"`.
  """

  name = _messages.StringField(1, required=True)


class CloudtraceProjectsTraceSinksListRequest(_messages.Message):
  r"""A CloudtraceProjectsTraceSinksListRequest object.

  Fields:
    pageSize: Optional. The maximum number of results to return from this
      request. Non-positive values are ignored. The presence of
      `next_page_token` in the response indicates that more results might be
      available.
    pageToken: Optional. If present, then retrieve the next batch of results
      from the preceding call to this method. `page_token` must be the value
      of `next_page_token` from the previous response. The values of other
      method parameters should be identical to those in the previous call.
    parent: Required. The parent resource whose sinks are to be listed
      (currently only project parent resources are supported):
      "projects/[PROJECT_ID]"
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudtraceProjectsTraceSinksPatchRequest(_messages.Message):
  r"""A CloudtraceProjectsTraceSinksPatchRequest object.

  Fields:
    name: Required. The full resource name of the sink to update, including
      the parent resource and the sink identifier:
      "projects/[PROJECT_NUMBER]/traceSinks/[SINK_ID]" Example:
      `"projects/12345/traceSinks/my-sink-id"`.
    traceSink: A TraceSink resource to be passed as the request body.
    updateMask: Required. Field mask that specifies the fields in `trace_sink`
      that are to be updated. A sink field is overwritten if, and only if, it
      is in the update mask. `name` and `writer_identity` fields cannot be
      updated. An empty `update_mask` is considered an error. For a detailed
      `FieldMask` definition, see https://developers.google.com/protocol-
      buffers/docs/reference/google.protobuf#fieldmask Example:
      `updateMask=output_config`.
  """

  name = _messages.StringField(1, required=True)
  traceSink = _messages.MessageField('TraceSink', 2)
  updateMask = _messages.StringField(3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ListTraceSinksResponse(_messages.Message):
  r"""Result returned from `ListTraceSinks`.

  Fields:
    nextPageToken: A paginated response where more pages might be available
      has `next_page_token` set. To get the next set of results, call the same
      method again using the value of `next_page_token` as `page_token`.
    sinks: A list of sinks.
  """

  nextPageToken = _messages.StringField(1)
  sinks = _messages.MessageField('TraceSink', 2, repeated=True)


class OpenTelemetryFormat(_messages.Message):
  r"""OpenTelemetryFormat contains metadata to help convert Cloud Trace trace
  format to OpenTelemetry trace format.

  Fields:
    version: Optional. OpenTelemetry format defined by
      https://github.com/open-telemetry/opentelemetry-proto/blob/main/opentele
      metry/proto/collector/trace/v1/trace_service.proto Version of the
      OpenTelemetry schema as it appears in the defining proto package.
      Currently, only "v1" is supported, but support for more version may
      added in the future.
  """

  version = _messages.StringField(1)


class OutputConfig(_messages.Message):
  r"""OutputConfig contains a destination for writing trace data.

  Fields:
    bigqueryConfig: Optional. Additional options governing the export behavior
      when the selected destination is a BigQuery dataset.
    destination: Required. The destination for writing trace data. Supported
      formats include:
      "bigquery.googleapis.com/projects/[PROJECT_ID]/datasets/[DATASET]"
    pubsubConfig: Optional. Additional options governing the export behavior
      when the selected destination is a Pub/Sub queue.
  """

  bigqueryConfig = _messages.MessageField('BigQueryOutputConfig', 1)
  destination = _messages.StringField(2)
  pubsubConfig = _messages.MessageField('PubsubOutputConfig', 3)


class PubsubOutputConfig(_messages.Message):
  r"""Configuration for the output that is specific to Pub/Sub when choosing a
  Pub/Sub queue as the output destination.

  Fields:
    openTelemetryFormat: open_telemetry_format contains additional information
      needed to convert Cloud Trace data.
  """

  openTelemetryFormat = _messages.MessageField('OpenTelemetryFormat', 1)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class TraceSink(_messages.Message):
  r"""Describes a sink used to export traces to a BigQuery dataset. The sink
  must be created within a project.

  Fields:
    name: Identifier. The canonical sink resource name, unique within the
      project. Must be of the form:
      projects/[PROJECT_NUMBER]/traceSinks/[SINK_ID]. E.g.:
      `"projects/12345/traceSinks/my-project-trace-sink"`. Sink identifiers
      are limited to 256 characters and can include only the following
      characters: upper and lower-case alphanumeric characters, underscores,
      hyphens, and periods.
    outputConfig: Required. The export destination.
    writerIdentity: Output only. A service account name for exporting the
      data. This field is set by sinks.create and sinks.update. The service
      account will need to be granted write access to the destination
      specified in the output configuration, see [Granting access for a
      resource](/iam/docs/granting-roles-to-service-
      accounts#granting_access_to_a_service_account_for_a_resource). To create
      tables and to write data, this account needs the `dataEditor` role. Read
      more about roles in the [BigQuery
      documentation](https://cloud.google.com/bigquery/docs/access-control).
      E.g.: "service-00000001@00000002.iam.gserviceaccount.com"
  """

  name = _messages.StringField(1)
  outputConfig = _messages.MessageField('OutputConfig', 2)
  writerIdentity = _messages.StringField(3)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command to install/update gcloud components."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.components import util


@base.UniverseCompatible
class Install(base.SilentCommand):
  """Install one or more Google Cloud CLI components.

  Ensure that each of the specified components (as well as any dependent
  components) is installed on the local workstation.  Components are installed
  without performing any upgrades to your existing CLI installation.  All
  components are installed at the current version of your CLI.
  """
  detailed_help = {
      'DESCRIPTION': """\
          {description}

          Components that are available for installation can be viewed by
          running:

            $ {parent_command} list

          Installing a given component will also install all components on which
          it depends.  The command lists all components it is about to install,
          and asks for confirmation before proceeding.

          ``{command}'' installs components from the version of the Google Cloud
          CLI you currently have installed.  You can see your current version by
          running:

            $ {top_command} version

          If you want to update your Google Cloud CLI installation to the latest
          available version, use:

            $ {parent_command} update
      """,
      'EXAMPLES': """\
          The following command installs ``COMPONENT-1'', ``COMPONENT-2'',
          and all components that they depend on:

            $ {command} COMPONENT-1 COMPONENT-2
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'component_ids',
        metavar='COMPONENT-IDS',
        nargs='+',
        help='The IDs of the components to be installed.')
    parser.add_argument(
        '--compile-python',
        required=False,
        hidden=True,
        default='True',
        action='store_true',
        help='THIS ARGUMENT NEEDS HELP TEXT.')

  def Run(self, args):
    """Runs the list command."""
    update_manager = util.GetUpdateManager(args)
    update_manager.Install(args.component_ids)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`gcloud components update-macos-python` command."""


from googlecloudsdk.calliope import base
from googlecloudsdk.core.updater import python_manager


@base.Hidden
@base.UniverseCompatible
class UpdateMacosPython(base.Command):
  """Updates managed Python installation on macOS."""

  @staticmethod
  def Args(parser):
    # Note: this command doesn't define any flags, and new flags should not be
    # added here. This is because update_manager.Update() forms the command line
    # to invoke this command using the existing gcloud code, then shells out to
    # the command, which runs using the new gcloud code. If the old and new
    # versions of the command had different sets of flags, this could result in
    # an error when parsing arguments (this is especially problematic for
    # version downgrades; any new flags added to this command would potentially
    # be missing when downgrading to older gcloud versions).
    pass

  def Run(self, args):
    python_manager.InstallPythonAndDependenciesOnMac()



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list Airflow plugins for a Cloud Composer environment."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.composer import resource_args
from googlecloudsdk.command_lib.composer import storage_util

DETAILED_HELP = {
    'EXAMPLES':
        """\
        To list the plugins for the Cloud Composer environment
        ``environment-1'' and location ``us-central1'', run:

          $ {command} --environment=environment-1 --location=us-central1
        """
}


class List(base.Command):
  """List the plugins for a Cloud Composer environment."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    resource_args.AddEnvironmentResourceArg(
        parser, 'for which to list plugins.', positional=False)
    parser.display_info.AddFormat('table(name)')

  def Run(self, args):
    env_ref = args.CONCEPTS.environment.Parse()
    return storage_util.List(
        env_ref, 'plugins', release_track=self.ReleaseTrack())



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show metadata for a specified project."""


from googlecloudsdk.api_lib.billing import billing_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.billing import flags
from googlecloudsdk.command_lib.billing import utils


class Describe(base.DescribeCommand):
  """Show detailed billing information for a project."""

  detailed_help = {
      'DESCRIPTION': """\
          This command shows billing info for a project, given its ID.

          This call can fail for the following reasons:

          * The project specified does not exist.
          * The active user does not have permission to access the given
            project.
          """,
      'EXAMPLES': """\
          To see detailed billing information for a project `my-project`, run:

              $ {command} my-project
          """
  }

  @staticmethod
  def Args(parser):
    flags.GetProjectIdArgument().AddToParser(parser)

  def Run(self, args):
    client = billing_client.ProjectsClient()
    project_ref = utils.ParseProject(args.project_id)
    return client.Get(project_ref)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of gcloud Procurement consumer orders describe command."""


from googlecloudsdk.api_lib.commerce_procurement import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.commerce_procurement import resource_args


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Describe(base.DescribeCommand):
  """Returns the Order object resulting from the Get API."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: argparse.ArgumentParser to register arguments with.
    """
    resource_args.AddOrderResourceArg(parser, 'Order to describe.')

  def Run(self, args):
    """Runs the command.

    Args:
      args: The arguments that were provided to this command invocation.

    Returns:
      An Order message.
    """
    order_ref = args.CONCEPTS.order.Parse()
    return apis.Orders.Get(order_ref.RelativeName())
